//---------------------------------------------------------------------
//
//							 Software License Agreement
//
// The software supplied herewith by Microchip Technology Incorporated 
// (the Company) for its PICmicro Microcontroller is intended and 
// supplied to you, the Companys customer, for use solely and 
// exclusively on Microchip PICmicro Microcontroller products. The 
// software is owned by the Company and/or its supplier, and is 
// protected under applicable copyright laws. All rights are reserved. 
//  Any use in violation of the foregoing restrictions may subject the 
// user to criminal sanctions under applicable laws, as well as to 
// civil liability for the breach of the terms and conditions of this 
// license.
//
// THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES, 
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
// TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
// PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
// IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
//
//---------------------------------------------------------------------
//	File:		Blind.c
//
//	Written By:	Jorge Zambada, Microchip Technology
//						
// 
// The following files should be included in the MPLAB project:
//
//		Blind.c       -- Main source code file
//		p30f3010.gld  -- Linker script file
//				
// CONFIGURATION FUSE INSTRUCTIONS:
// Osc MODE: XT w/PLL 8X
// WDT: disabled
// Comm Channel Select: Use EMUC2 and EMUD2
//---------------------------------------------------------------------
//
// Revision History
//
// 6/19/05 -- first version 
// 8/05/05 -- second version 
//----------------------------------------------------------------------
//
// Blind.c shows how a BLDC motor can be spun simply by enabling two 
// winding at a time over a periodic time interval.  In this case a timer
// interrupt is setup every 2 mS. In that interrupt a lookup table is 
// referenced and two of the three windings are energised. No care is taken
// to see if the energised windings are aligned to the rotor magnets. The 
// efficiency is very low, the motor runs rough, draws a lot of current and
// the motor heats up. This experiment is purely to demonstrate the need to
// align the motor windings to the rotor magnets which will be done in Lab 3

// 	Motor Terminal		|	MCLV Connection
// ---------------------|-------------------
//	White Phase --------|-- M3
//	Black Phase --------|-- M2
//	Red Phase   --------|-- M1
//  Green Wire  --------|--	G
//  Hall Supply --------|-- +5V
//  Hall Ground --------|-- GND
//	White Hall  --------|-- HA
//	Brown Hall  --------|-- HB
//	Green Hall  --------|-- HC

#include "p30F3010.h"

#define FCY  10000000
#define MILLISEC FCY/10000  // 1 mSec delay constant
#define S2 PORTCbits.RC14

unsigned char SSTIndex = 0;		// Pointer to the Six Step Table
void DelayNmSec(unsigned int N);// Delay Subroutine
void InitTMR1(void);			// Initialize Timer 1 to commutate motor

// This table contains the transistor states for each sector the motor is in
const unsigned int SixStepTable[] = 
                   {0x0021, /* PWM3H PWM1L */
                    0x0024, /* PWM3H PWM2L */
                    0x0006, /* PWM1H PWM2L */
                    0x0012, /* PWM1H PWM3L */
                    0x0018, /* PWM2H PWM3L */
                    0x0009};/* PWM2H PWM1L */

//---------------------------------------------------------------------
// 2 mSec Interrupt vector for Timer 1
void __attribute__((__interrupt__)) _T1Interrupt(void)
{
	IFS0bits.T1IF = 0;
	OVDCON = (int)SixStepTable[SSTIndex++];
	if (SSTIndex == 6)
		SSTIndex = 0;
	return;		
}

//---------------------------------------------------------------------
// main subroutine. Initializes all modules used and turns on and off
// timer 1
int main(void)	
{
	ADPCFG = 0xFFF8;		// all PORTB = Digital;RB0 to RB2 = analog
	PWMCON1 = 0x0F77;		// Enable PWM outputs
	PTCONbits.PTEN = 1;		// Turn on PWM module
	InitTMR1();				// Initialize 2 ms ISR
	while(1)
	{
		OVDCON = 0;			// Turn off every transistor
		while (S2);			// wait till S2 is pressed 
		DelayNmSec(30);		// -- do --
		while (!S2);		// if pressed wait till S2 is released
		DelayNmSec(30);		// -- do --
		T1CONbits.TON = 1;	// turn on T1
		while (S2);			// wait till S2 is pressed 
		DelayNmSec(30);		// -- do --
		while (!S2);		// if pressed wait till S2 is released
		DelayNmSec(30);		// -- do --
		T1CONbits.TON = 0;	// turn on T1
	}
	return;
}

//---------------------------------------------------------------------
// This is a generic 1ms delay routine to give a 1mS to 65.5 Seconds delay
// For N = 1 the delay is 1 mS, for N = 65535 the delay is 65,535 mS. 
// Note that FCY is used in the computation.  Please make the necessary
// Changes(PLLx4 or PLLx8 etc) to compute the right FCY as in the define
// statement above.
void DelayNmSec(unsigned int N)
{
	unsigned int j;
	while(N--)
	 	for(j=0;j < MILLISEC;j++);
	return;
}

//---------------------------------------------------------------------
// Initialize timer 1. Period of the ISR is 2 ms for this example
void InitTMR1(void)
{
	TMR1 = 0x0000;			// clr tmr1
	PR1 = 20000;			// period for 2 mS
	T1CON = 0x0000;			// 1:1 Prescale Value
	IFS0bits.T1IF = 0;		//clear interrupt flag
	IEC0bits.T1IE = 1;		// enable T1 interrutps
	return;
}

