//---------------------------------------------------------------------
//
//							 Software License Agreement
//
// The software supplied herewith by Microchip Technology Incorporated 
// (the Company) for its PICmicro Microcontroller is intended and 
// supplied to you, the Companys customer, for use solely and 
// exclusively on Microchip PICmicro Microcontroller products. The 
// software is owned by the Company and/or its supplier, and is 
// protected under applicable copyright laws. All rights are reserved. 
//  Any use in violation of the foregoing restrictions may subject the 
// user to criminal sanctions under applicable laws, as well as to 
// civil liability for the breach of the terms and conditions of this 
// license.
//
// THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES, 
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
// TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
// PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
// IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
//
//---------------------------------------------------------------------
//	File:		BldcSen.c
//
//	Written By:	Jorge Zambada, Microchip Technology
//						
// 
// The following files should be included in the MPLAB project:
//
//		BldcSen.c		-- Main source code file
//		p30f3010.gld	-- Linker script file
//				
// CONFIGURATION FUSE INSTRUCTIONS:
// Osc MODE: XT w/PLL 8X
// WDT: disabled
// Comm Channel Select: Use EMUC2 and EMUD2
//---------------------------------------------------------------------
//
// Revision History
//
// 6/19/05 -- first version 
// 8/05/05 -- second version 
//----------------------------------------------------------------------
//
// BldcSen.c uses the hall effect sensors on the BLDC motor
// to sense the position of the rotor magnets w.r.t the motor windings
// Based on the rotor position, the appropriate windings are excited 
// This method improves the efficiency of the motor to a great degree
// The motor rotates very smoothly, draws less current and runs 
// without heating. Note that the motor is running at full speed.

// 	Motor Terminal		|	MCLV Connection
// ---------------------|-------------------
//	White Phase --------|-- M3
//	Black Phase --------|-- M2
//	Red Phase   --------|-- M1
//  Green Wire  --------|--	G
//  Hall Supply --------|-- +5V
//  Hall Ground --------|-- GND
//	White Hall  --------|-- HA
//	Brown Hall  --------|-- HB
//	Green Hall  --------|-- HC

#include "p30F3010.h"

#define FCY  10000000
#define MILLISEC FCY/10000  // 1 mSec delay constant
#define S2 PORTCbits.RC14

void DelayNmSec(unsigned int N);// Delay Subroutine

const unsigned int SixStepTable[] = 
                   {0x0000, /* Invalid! */
                    0x0021, /* PWM3H PWM1L */
                    0x0018, /* PWM2H PWM3L */
                    0x0009, /* PWM2H PWM1L */
                    0x0006, /* PWM1H PWM2L */
                    0x0024, /* PWM3H PWM2L */
                    0x0012, /* PWM1H PWM3L */
                    0x0000};/* Invalid! */

unsigned int HallValue;		// Variable containing the Hall Value from PORTB

int main(void)
{
	ADPCFG = 0xFFF8;		// all PORTB = Digital;RB0 to RB2 = analog
	PWMCON1 = 0x0777;		// Enable PWM outputs
	PTCONbits.PTEN = 1;		// Turn on PWM module
	while (1)
	{
		OVDCON = 0x0000;	// Turn off every transistor
		while (S2);			// wait till S2 is pressed 
		DelayNmSec(30);		// -- do --
		while (!S2);		// if pressed wait till S2 is released
		DelayNmSec(30);		// -- do --

		while (S2)
		{
			HallValue = PORTB & 0x0038; 			// Read Port
			HallValue = HallValue >> 3; 			// Shift Right
			OVDCON = (int)SixStepTable[HallValue];	// Commutate BLDC
		}

		OVDCON = 0x0000;    // Turn off every transistor
		DelayNmSec(30);		// -- do --
		while (!S2);		// if pressed wait till S2 is released
		DelayNmSec(30);		// -- do --

	}
	return;
}

//---------------------------------------------------------------------
// This is a generic 1ms delay routine to give a 1mS to 65.5 Seconds delay
// For N = 1 the delay is 1 mS, for N = 65535 the delay is 65,535 mS. 
// Note that FCY is used in the computation.  Please make the necessary
// Changes(PLLx4 or PLLx8 etc) to compute the right FCY as in the define
// statement above.

void DelayNmSec(unsigned int N)
{
	unsigned int j;
	while(N--)
	 	for(j=0;j < MILLISEC;j++);
	return;
}
