//---------------------------------------------------------------------
//
//							 Software License Agreement
//
// The software supplied herewith by Microchip Technology Incorporated 
// (the Company) for its PICmicro Microcontroller is intended and 
// supplied to you, the Companys customer, for use solely and 
// exclusively on Microchip PICmicro Microcontroller products. The 
// software is owned by the Company and/or its supplier, and is 
// protected under applicable copyright laws. All rights are reserved. 
//  Any use in violation of the foregoing restrictions may subject the 
// user to criminal sanctions under applicable laws, as well as to 
// civil liability for the breach of the terms and conditions of this 
// license.
//
// THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES, 
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
// TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
// PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
// IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
//
//---------------------------------------------------------------------
//	File:		OpenLoopSenBLDC.c
//
//	Written By:	Jorge Zambada, Microchip Technology
//						
// 
// The following files should be included in the MPLAB project:
//
//		OpenLoopSenBLDC.c -- Main source code file
//		p30f3010.gld      -- Linker script file
//				
// CONFIGURATION FUSE INSTRUCTIONS:
// Osc MODE: XT w/PLL 8X
// WDT: disabled
// Comm Channel Select: Use EMUC2 and EMUD2				
//---------------------------------------------------------------------
//
// Revision History
//
// 10/01/04 -- first version 
// 8/05/05  -- second version 
//---------------------------------------------------------------------- 
// OpenLoopSenBlDC.c uses the periphals on the dsPIC to implement a 
// sensored open loop BLDC motor control algorithm.
// In this program the Hall effect sensors are connected to the 
// Change Notification pins.  When a transition from one sector to 
// the next occurs, the value on the CN pins changes and causes an
// interrupt.  In this interrupt the Hall sensors are read, and used
// as an offset in a lookup table to generate a new 16-bit value
// for the OVDCON register.  This value enables windings at PWM, low and
// inactive appropriately to drive the BLDC motor.

// 	Motor Terminal		|	MCLV Connection
// ---------------------|-------------------
//	White Phase --------|-- M3
//	Black Phase --------|-- M2
//	Red Phase   --------|-- M1
//  Green Wire  --------|--	G
//  Hall Supply --------|-- +5V
//  Hall Ground --------|-- GND
//	White Hall  --------|-- HA
//	Brown Hall  --------|-- HB
//	Green Hall  --------|-- HC

#include "p30F3010.h"

#define FCY  10000000		// xtal = 5.0Mhz; PLLx8
#define MILLISEC FCY/10000	// 1 mSec delay constant
#define FPWM 19531			// Value used to simplify math on the Duty
							// Cycle registers calculation
#define S2	PORTCbits.RC14

void DelayNmSec(unsigned int N); // General Delay subroutine
void InitMCPWM(void);			// Init PWM Module to drive the Inverter
void InitADC10(void);		   // A/D Init Subroutine to Read POT.
void InitCN(void);			  // Change Notification for Hall Effect Sensors

unsigned int HallValue;		// Variable containing the Hall Value from PORTB

/*************************************************************
	Low side driver table is as below.  In the StateLoTableClk
	and the StateLoTableAntiClk tables, the Low side driver is
	PWM while the high side driver is either on or off.  
*************************************************************/

unsigned int StateLoTable[] = {0x0000,  /* all PWM OFF */
                               0x2001,  /* PWM1L -> 1, PWM3H -> PWM*/
                               0x0810,  /* PWM3L -> 1, PWM2H -> PWM*/
                               0x0801,  /* PWM1L -> 1, PWM2H -> PWM*/
                               0x0204,  /* PWM2L -> 1, PWM1H -> PWM*/
                               0x2004,  /* PWM2L -> 1, PWM3H -> PWM*/
                               0x0210,  /* PWM3L -> 1, PWM1H -> PWM*/
                               0x0000}; /* all PWM OFF */

/****************************************************************
Interrupt vector for Change Notification CN5, 6 and 7 is as below. When a 
Hall sensor changes states, an interrupt will be caused which will vector 
to the routine below. The program then reads PORTB, mask bits 3, 4 and 5,
shift and adjust the value to read as 1, 2 ... 6.  This value is then used
as an offset in the lookup table StateLoTableClk or StateLoTableAntiClk to
determine the value loaded in the OCDCON register.
*****************************************************************/ 

void __attribute__((__interrupt__)) _CNInterrupt (void)
{
	IFS0bits.CNIF = 0;					// clear flag
	HallValue = PORTB & 0x0038;			// mask RB3,4 & 5
	HallValue = HallValue >> 3;			// shift right 3 times
	OVDCON = StateLoTable[HallValue];	// Energize Transistors from Table
	return;
}

/*********************************************************************
The ADC interrupt loads the PDCx registers with the demand pot
value.  This is only done when the motor is running.
*********************************************************************/

void __attribute__((__interrupt__)) _ADCInterrupt (void)
{
	IFS0bits.ADIF = 0;	// Clear Interrupt Flag
	PDC1 = ADCBUF0;		// get value ...
	PDC2 = PDC1;		// and load all three PWMs ...
	PDC3 = PDC1;		// duty cycles
}

/*********************************************************************
main subroutine. It initialized the peripherals used in the application,
* Input Change Notifications for Hall effect sensores,
* MCPWM module for driving the inverter,
* A/D Converter for reading the POT value.
When S2 is pressed, the hall effect sensors are read the the corresponding
transistors are turned on depending on a look up table. After energizing the
first pair of transistors everything else is done in the Change Notification
interrupts.
*********************************************************************/

int main(void)
{
	InitCN();		// Init CN for Hall effect sensor inputs
	InitMCPWM();	// Init PWM for 19.5 kHz operation
	InitADC10();	// Init ADC for POT measurement
	while(1)
	{		
		while (S2);							// wait for start key hit
		DelayNmSec(30);						// Debounce
		while (!S2);						// wait till key is released
		DelayNmSec(30);						// Debounce
		HallValue = PORTB & 0x0038;			// Read Hall Effect Sensors
		HallValue = HallValue >> 3;			// shift right 3 bits
		OVDCON = StateLoTable[HallValue];	// Load Commutation Value
		PWMCON1 = 0x0777;					// enable PWM outputs
		while(S2);							// wait for start key hit
		DelayNmSec(30);						// Debounce
		while(!S2);							// wait till key is released
		DelayNmSec(30);						// Debounce
		PWMCON1 = 0x0700;					// disable PWM outputs
		OVDCON = 0x0000;					// overide PWM low.
	}
	return;
}
	
/*******************************************************************
Below is the code required to setup the ADC registers for :
1. 1 channel conversion (in this case RB2/AN2)
2. PWM trigger starts conversion
3. Pot is connected to CH0 and RB2
4. Manual Stop Sampling and start converting
5. Manual check of Conversion complete
*********************************************************************/

void InitADC10(void)
{
	ADPCFG = 0xFFF8;	// all PORTB = Digital;RB0 to RB2 = analog
	ADCON1 = 0x0064;	// PWM starts conversion
	ADCON2 = 0x0200;	// simulataneous sample 4 channels									
	ADCHS = 0x0002;		// Connect RB2/AN2 as CH0 = pot  ..
						// ch1 = Vbus, Ch2 = Motor, Ch3 = pot
	ADCON3 = 0x0005;	// Tad = 3Tcy
	IFS0bits.ADIF = 0;
	IEC0bits.ADIE = 1;
	
	ADCON1bits.ADON = 1;// turn ADC ON
	
	return;
}

/********************************************************************
InitMCPWM, intializes the PWM as follows:
1. FPWM = 19531 hz
2. Independant PWMs
3. Control outputs using OVDCON
4. Set Duty Cycle with the ADC value read from pot
5. Set ADC to be triggered by PWM special trigger
*********************************************************************/

void InitMCPWM(void)
{
	PTPER = FCY/FPWM - 1;
	PWMCON1 = 0x0700;	// disable PWMs
	OVDCON = 0x0000;	// allow control using OVD
	PDC1 = 0;			// init PWM 1, 2 and 3 to 0%
	PDC2 = 0;
	PDC3 = 0;
	SEVTCMP = PTPER;
	PWMCON2 = 0x0F00;	// 16 postscale values
	PTCON = 0x8000;		// start PWM
	return;
}

/********************************************************************
InitCN, Initializes the Input Change Notification for the Hall Effect
Sensors
*********************************************************************/

void InitCN(void)
{
	CNEN1 = 0x00E0;		// Enable CN5, CN6 and CN7
	IFS0bits.CNIF = 0;	// clear CNIF
	IEC0bits.CNIE = 1;	// enable CN interrupt
	return;
}

//---------------------------------------------------------------------
// This is a generic 1ms delay routine to give a 1mS to 65.5 Seconds delay
// For N = 1 the delay is 1 mS, for N = 65535 the delay is 65,535 mS. 
// Note that FCY is used in the computation.  Please make the necessary
// Changes(PLLx4 or PLLx8 etc) to compute the right FCY as in the define
// statement above.

void DelayNmSec(unsigned int N)
{
	unsigned int j;
	while(N--)
	 	for(j=0;j < MILLISEC;j++);
	return;
}
