//---------------------------------------------------------------------
//
//							 Software License Agreement
//
// The software supplied herewith by Microchip Technology Incorporated 
// (the Company) for its PICmicro Microcontroller is intended and 
// supplied to you, the Companys customer, for use solely and 
// exclusively on Microchip PICmicro Microcontroller products. The 
// software is owned by the Company and/or its supplier, and is 
// protected under applicable copyright laws. All rights are reserved. 
//  Any use in violation of the foregoing restrictions may subject the 
// user to criminal sanctions under applicable laws, as well as to 
// civil liability for the breach of the terms and conditions of this 
// license.
//
// THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES, 
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
// TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
// PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
// IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
// CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
//
//---------------------------------------------------------------------
//	File:		PhaseAdvance.c
//
//	Written By:	Steve Bowling & Jorge Zambada, Microchip Technology
// 
//  The following files should be included in the MPLAB project:
//
//		PhaseAdvance.c		    -- Main source code file
//		p30f3010.gld		-- Linker script file
//
//---------------------------------------------------------------------
//
// Revision History
//
// 05/02/05 -- first version 
// 08/17/05 -- second version 
//---------------------------------------------------------------------- 

#include <p30f3010.h>

//--------------------------Device Configuration------------------------
_FOSC(CSW_FSCM_OFF & XT_PLL16);
_FWDT(WDT_OFF);
_FBORPOR(PBOR_ON & BORV_20 & PWRT_64 & MCLR_EN);
//----------------------------------------------------------------------

typedef signed int SFRAC16;

#define FCY  20000000	// xtal = 5Mhz; PLLx16 -> 20 MIPS
#define FPWM 19531		// 19.5 kHz, so that no audible noise is present.
#define _10MILLISEC		10
#define _50MILLISEC		50
#define _100MILLISEC	100
#define _500MILLISEC	500

#define HALLA	1	// RB3
#define HALLB	2	// RB4
#define HALLC	4	// RB5
#define CW	0		// Actual Hurst Motor speed is CCW
#define CCW	1		// Actual Hurst Motor speed is CW
#define Switch_2	(!PORTCbits.RC14)

// Period Calculation
// Period = TMRClock * 6 / RPM
// For example>
// RPM = 6000 (Max Speed)
// Period = (20,000,000 / 64) * 6 / 6000 = 312.5
// RPM = 60 (Min Speed)
// Period = (20,000,000 / 64) * 6 / 60 = 31250

#define MINPERIOD	313		// For 6000 max rpm and 10 poles motor
#define MAXPERIOD	31250	// For 60 min rpm and 10 poles motor
#define MIN_PER_PH_ADV	(MAXPERIOD)
#define MINABSSPEED	327
#define MAXABSSPEED 32663
#define MIN_SPD_PH_ADV	(MINABSSPEED * 2)
#define _500RPMs	2722

// Use this MACRO when using floats to initialize signed 16-bit fractional variables
#define SFloat_To_SFrac16(Float_Value)	(((Float_Value) < 0.0) ? (SFRAC16)(32768 * (Float_Value) - 0.5) : (SFRAC16)(32767 * (Float_Value) + 0.5))

void SixStepComm(int _Current_Sector, int _Required_Voltage);	// This subroutine is used for commutating the motor depending on its rotor position
void PhaseAdvance(void);	// It is used for configuring the timer that will call the commutation subroutine when phase advance is enabled
void InitADC10(void);	// Initialization of ADC used for Speed Command
void InitMCPWM(void);	// Initialization for PWM at 20kHz, Center aligned, Complementary mode with 500 ns of deadtime
void InitTMR1(void);	// Initialization for TIMER1 used for speed control and motor stalled protection
void InitTMR2(void);	// Initialization for TIMER2 used for Phase Advance control
void InitTMR3(void);	// Initialization for TIMER3 used as a timebase for the two input capture channels
void InitUserInt(void);	// This function initializes all ports (inputs and outputs) for the application
void Init_IC_and_CN(void);	// Initializes input captures and change notification, used for the hall sensor inputs
void RunMotor(void);	// This function initializes all variables and interrupts used for starting and running the motor
void StopMotor(void);	// This function clears all flags, and stops anything related to motor control, and also disables PWMs
void ForceCommutation(void);	// When motor is to slow to generate interrupts on halls, this function forces a commutation
void ChargeBootstraps(void);	// At the begining of the motor operation, the bootstrap caps are charged with this function
extern void SpeedControl(void);	// This function contains all ASM and C operations for doing the PID Control loop for the speed
extern void SpeedCalculation(void);
extern void Phase_Adv_Profile(void);

// Flags used for the application
struct 
{
	unsigned MotorRunning	:1;
	unsigned MotorCommInAdv :1;
	unsigned MotorAdvEnabled:1;
	unsigned unused			:13;
}Flags;
 
unsigned int HallValue;	// This variable holds the hall sensor input readings
unsigned int Sector;	// This variables holds present sector value. Which is the rotor position
unsigned int LastSector;	// This variable holds the last sector value. This is critical to filter halls slew rate HW issue
unsigned int MotorStalledCounter = 0;	// This variable gets incremented each 1 ms, and is cleared everytime a new sector is
										// detected. Used for ForceCommutation and MotorStalled protection functions

// 	Motor Terminal		|	MCLV Connection
// ---------------------|-------------------
//	White Phase --------|-- M3
//	Black Phase --------|-- M2
//	Red Phase   --------|-- M1
//  Green Wire  --------|--	G
//  Hall Supply --------|-- +5V
//  Hall Ground --------|-- GND
//	White Hall  --------|-- HA
//	Brown Hall  --------|-- HB
//	Green Hall  --------|-- HC

// This array translates the hall state value read from the digital
// I/O to the proper commutation sector.  Hall values of 0 or 7 
// represent illegal values and therefore return -1.
// Motor Connection for this Sector Table:

char SectorTable[] = {-1,4,2,3,0,5,1,-1};

/*************************************************************
	Low side driver table is as below.  The high side driver is
	PWM while the low side driver is either on or off.
*************************************************************/

unsigned int StateLoTable[] = {	0x0204,  /* PWM2L -> 1, PWM1H -> PWM */
							 	0x0210,  /* PWM3L -> 1, PWM1H -> PWM */
                                0x0810,  /* PWM3L -> 1, PWM2H -> PWM */
								0x0801,  /* PWM1L -> 1, PWM2H -> PWM */
								0x2001,  /* PWM1L -> 1, PWM3H -> PWM */
								0x2004}; /* PWM2L -> 1, PWM3H -> PWM */

/* Phase Advance Variables */

unsigned int Phase_Advance_Perc_Dividend;
unsigned int Phase_Advance_Perc_Divisor;
unsigned int Phase_Advance_Value;	// Value to store in the period register of the timer doing the commutation when phase advance enabled

/* Speed Control Variables */

unsigned char Current_Direction;	// Current mechanical motor direction of rotation Calculated in halls interrupts
unsigned char Required_Direction;	// Required mechanical motor direction of rotation, will have the same sign as the
									// ControlOutput variable from the Speed Controller
unsigned int PastCapture, ActualCapture, Period;	// Variables containing the Period of half an electrical cycle, which is
													// an interrupt each edge of one of the hall sensor input
SFRAC16 Speed, RefSpeed;	// Actual and Desired speeds for the PID controller, that will generate the error
SFRAC16 SpeedAbsValue;		// Absolute value of the Speed variable. Used by the phase advance subroutine
SFRAC16 ControlOutput = 0;	// Controller output, used as a voltage output, use its sign for the required direction
// Constants used by the PID controller, since a MAC operation is used, the PID structure is changed (See SpeedControl() Comments

// Used as a temporal variable to perform a fractional divide operation in assembly
SFRAC16 _MINPERIOD = MINPERIOD - 1;	// Used by fractional divide operation

/**********************************************************************
	Timer 1 interrupt, In this ISR the Period and Speed	are calculated 
    based on the input capture of one of the halls.
	The speed controller is also called in this ISR to generate a new
	output voltage (ControlOutput). The last thing done in this ISR is
	the forced commutation, which happens each time the motor doesn't
	generate a new hall interrupt after a programmed period of time.
	If the timeout for generating hall ISR is too much (i.e. 100 ms)
	the motor is then stopped.
	Note: The Speed Calculation is made in assembly to take advantage of
	the signed fractional division.
**********************************************************************/

void __attribute__((__interrupt__)) _T1Interrupt (void)
{
	IFS0bits.T1IF = 0;
	Period = ActualCapture - PastCapture;	// This is an UNsigned substraction

	if (Period < (unsigned int)MINPERIOD) 	// MINPERIOD corresponds to 6000 rpm
		Period = MINPERIOD;
	else if (Period > (unsigned int)MAXPERIOD) // MAXPERIOD corresponds to 60 rpm
		Period = MAXPERIOD;

	// This subroutine in assembly calculates the Speed using fractional division
	//	                              MINPERIOD
	// Speed = (Fractional divide) ---------------
	//                                 Period

	SpeedCalculation();
	SpeedAbsValue = Speed;

	// Speed sign adjustment based on current motor direction of rotation
	if (Current_Direction == CCW)
		Speed = -Speed;

	// Condition        RPM          SFRAC16      SINT      HEX
	// Max Speed CW  -> 6000 RPM  -> 0.996805  -> 32663  -> 0x7F97
	// Min Speed CW  -> 60 RPM    -> 0.009984  -> 327    -> 0x0147
	// Min Speed CCW -> -60 RPM   -> -0.009984 -> -327   -> 0xFEB9
	// Max Speed CCW -> -6000 RPM -> -0.996805 -> -32663 -> 0x8069

	// This Lab is Open loop, so that we can show the effects of phase advance.
	// The ControlOutput variable is overwritten with the reference speed.
	ControlOutput = RefSpeed;

	// Set minimum ControlOutput to 10% to keep the motor running
	if (ControlOutput < 3276)
		ControlOutput = 3276;

	MotorStalledCounter++;	// This variable is cleared in halls ISRs
    if ((MotorStalledCounter % _10MILLISEC) == 0)
	{
		ForceCommutation();	// Force Commutation if no hall sensor changes have occured in specified timeout.
	}
	else if (MotorStalledCounter >= _100MILLISEC)
	{
		StopMotor(); // Stop motor is no hall changes have occured in specified timeout
	}
	return;
}

/**********************************************************************
	Timer 2 interrupt is triggered by the PhaceAdvance subroutine in
	order to commutate the motor instead of doing this on the Hall sensor
	interrupts. This is because when phase advance, the commutation must
	lead the hall change interrupts, so a timer has to be used for this
	reason.
**********************************************************************/

void __attribute__((__interrupt__)) _T2Interrupt (void)
{
	IEC0bits.T2IE = 0;	// Disable Interrupts
	IFS0bits.T2IF = 0;	// Clear Flag
	T2CONbits.TON = 0;	// Turn Off timer

	if (Required_Direction == CW)
		SixStepComm((Sector + 1) % 6, ControlOutput);
	else	
		SixStepComm((Sector + 5) % 6, ControlOutput);
	
	Flags.MotorCommInAdv = 1;	// This flag is to let the hall sensor interrupts know that the commutation is already done by the timer 2
	
	return;
}

/**********************************************************************
	Hall A interrupt, this is done with the input change notification CN5.
	The purpose of this ISR is to Calculate the actual mechanical
	direction of rotation of the motor, and to adjust the Phase variable
	depending on the sector the rotor is in.
	Note: The sector must be validated in order to avoid any spurious
	interrupt due to a slow slew rate on the halls inputs due to hardware
	filtering.
**********************************************************************/

void __attribute__((__interrupt__)) _CNInterrupt (void)
{
	IFS0bits.CNIF = 0;	// Clear interrupt flag
	HallValue = (unsigned int)((PORTB >> 3) & 0x0007);	// Read halls
	Sector = SectorTable[HallValue];	// Get Sector from table
	if (Sector != LastSector)	// This is a MUST for getting around the HW slow rate
	{
		// Since a new sector is detected, clear variable that would stop the motor if stalled.
		MotorStalledCounter = 0;
		// Motor current direction is computed based on Sector
		if ((Sector == 5) || (Sector == 2))	// If going forward for falling and rising then CW
			Current_Direction = CCW;
		else
			Current_Direction = CW;
	
		if (Flags.MotorAdvEnabled == 1)
		{
			if (Flags.MotorCommInAdv == 0)	// If phase advance is enabled and no commutation has happened in the timer, then commutate here
				SixStepComm(Sector, ControlOutput); // If ControlOutput > 0 will run CW, else will run CCW

			PhaseAdvance();	// Configute timer 2 to trigger a commutation.
		}
		else
		{
			SixStepComm(Sector, ControlOutput); // If ControlOutput > 0 will run CW, else will run CCW
		}

		if (SpeedAbsValue > (unsigned int)MIN_SPD_PH_ADV)	// If motor fast enough, enabled phase advance
			Flags.MotorAdvEnabled = 1;
		else if (SpeedAbsValue < (unsigned int)(MIN_SPD_PH_ADV - _500RPMs))	// If motor is slower including hysteresis, then disable phase advance
			Flags.MotorAdvEnabled = 0;

		LastSector = Sector;	// If a change in sector, update last sector
	}
	return;
}

/**********************************************************************
	Hall B interrupt, this is done with the input change notification IC7.
	The purpose of this ISR is to Calculate the actual mechanical
	direction of rotation of the motor, and to adjust the Phase variable
	depending on the sector the rotor is in AND ALSO to calculate the
	mechanical Period between hall transition on the same input using
	input capture read.
	Note: The sector must be validated in order to avoid any spurious
	interrupt due to a slow slew rate on the halls inputs due to hardware
	filtering.
**********************************************************************/

void __attribute__((__interrupt__)) _IC7Interrupt (void)
{
	IFS1bits.IC7IF = 0;	// Cleat interrupt flag
	HallValue = (unsigned int)((PORTB >> 3) & 0x0007);	// Read halls
	Sector = SectorTable[HallValue];	// Get Sector from table
	if (Sector != LastSector)	// This is a MUST for getting around the HW slow rate
	{
		// Calculate Hall period corresponding to half an electrical cycle
		PastCapture = ActualCapture;
		ActualCapture = IC7BUF;
		// Clear all buffers
		IC7BUF;
		IC7BUF;
		IC7BUF;
		// Since a new sector is detected, clear variable that would stop the motor if stalled.
		MotorStalledCounter = 0;
		// Motor current direction is computed based on Sector
		if ((Sector == 3) || (Sector == 0))	// If going forward for falling and rising then CW
			Current_Direction = CCW;
		else
			Current_Direction = CW;

		if (Flags.MotorAdvEnabled == 1)
		{
			if (Flags.MotorCommInAdv == 0)	// If phase advance is enabled and no commutation has happened on the timer, then commutate here
				SixStepComm(Sector, ControlOutput); // If ControlOutput > 0 will run CW, else will run CCW

			PhaseAdvance();	// Configute timer 2 to trigger a commutation.
		}
		else
		{
			SixStepComm(Sector, ControlOutput); // If ControlOutput > 0 will run CW, else will run CCW
		}

		if (SpeedAbsValue > (unsigned int)MIN_SPD_PH_ADV)	// If motor fast enough, enabled phase advance
			Flags.MotorAdvEnabled = 1;
		else if (SpeedAbsValue < (unsigned int)(MIN_SPD_PH_ADV - _500RPMs))	// If motor is slower including hysteresis, then disable phase advance
			Flags.MotorAdvEnabled = 0;

		LastSector = Sector;	// If a change in sector, update last sector
	}
	return;
}

/**********************************************************************
	Hall C interrupt, this is done with the input change notification IC8.
	The purpose of this ISR is to Calculate the actual mechanical
	direction of rotation of the motor, and to adjust the Phase variable
	depending on the sector the rotor is in.
	Note: The sector must be validated in order to avoid any spurious
	interrupt due to a slow slew rate on the halls inputs due to hardware
	filtering.
**********************************************************************/

void __attribute__((__interrupt__)) _IC8Interrupt (void)
{	
	IFS1bits.IC8IF = 0;	// Cleat interrupt flag
	HallValue = (unsigned int)((PORTB >> 3) & 0x0007);	// Read halls
	Sector = SectorTable[HallValue];	// Get Sector from table
	if (Sector != LastSector)	// This is a MUST for getting around the HW slow rate
	{
		// Since a new sector is detected, clear variable that would stop the motor if stalled.
		MotorStalledCounter = 0;
		// Motor current direction is computed based on Sector
		if ((Sector == 1) || (Sector == 4))	// If going forward for falling and rising then CW
			Current_Direction = CCW;
		else
			Current_Direction = CW;
		
		if (Flags.MotorAdvEnabled == 1)
		{
			if (Flags.MotorCommInAdv == 0)	// If phase advance is enabled and no commutation has happened on the timer, then commutate here
				SixStepComm(Sector, ControlOutput); // If ControlOutput > 0 will run CW, else will run CCW

			PhaseAdvance();	// Configute timer 2 to trigger a commutation.
		}
		else
		{
			SixStepComm(Sector, ControlOutput); // If ControlOutput > 0 will run CW, else will run CCW
		}

		if (SpeedAbsValue > (unsigned int)MIN_SPD_PH_ADV)	// If motor fast enough, enabled phase advance
			Flags.MotorAdvEnabled = 1;
		else if (SpeedAbsValue < (unsigned int)(MIN_SPD_PH_ADV - _500RPMs))	// If motor is slower including hysteresis, then disable phase advance
			Flags.MotorAdvEnabled = 0;

		LastSector = Sector;	// If a change in sector, update last sector
	}
	return;
}

/*********************************************************************
	The ADC interrupt loads the reference speed (RefSpeed) with the 
	respective value of the POT. The value will be a unsigned fractional 
	value.
*********************************************************************/

void __attribute__((__interrupt__)) _ADCInterrupt (void)
{
	IFS0bits.ADIF = 0;
	RefSpeed = (int)(((unsigned int)ADCBUF0) / 2);
	if (RefSpeed < 2000) 
		RefSpeed = 2000;
	return;
}

/*********************************************************************
	main function of the application. Peripherals are initialized,
	and then, depending on the motor status (running or stopped) and
	if the push button is pressed, the motor is started or stopped.
	All other operations and state machines are performed with
	interrupts.
*********************************************************************/

int main(void)
{
	InitUserInt();	// Initialize User Interface I/Os
	InitADC10();	// Initialize ADC to be signed fractional
	InitTMR1();		// Initialize TMR1 for 1 ms periodic ISR
	InitTMR2();		// Initialize TMR2 for Phase advance
	InitTMR3();		// Initialize TMR3 for timebase of capture
	Init_IC_and_CN();	// Initialize Hall sensor inputs ISRs	
	InitMCPWM();	// Initialize PWM @ 20 kHz, center aligned, 500 ns of deadtime

	for(;;)
	{
		if ((Switch_2) && (!Flags.MotorRunning))
		{
			while(Switch_2);
			RunMotor();	// Run motor if push button is pressed and motor is stopped
		}
		else if ((Switch_2) && (Flags.MotorRunning))
		{
			while(Switch_2);
			StopMotor();	// Stop motor if push button is pressed and motor is running
		}
	}
	return 0;
}

/*********************************************************************
	In the topology used, it is necessary to charge the bootstrap
	caps each time the motor is energized for the first time after an
	undetermined amount of time. ChargeBootstraps subroutine turns ON
	the lower transistors for 10 ms to ensure voltage on these caps,
	and then it transfers the control of the outputs to the PWM
	module.
*********************************************************************/

void ChargeBootstraps(void)
{
	unsigned int i;
	OVDCON = 0x0015;	// Turn ON low side transistors to charge
	for (i = 0; i < 33330; i++) // 10 ms Delay at 20 MIPs
		;
	OVDCON = 0x0000;
	return;
}

/*********************************************************************
	Call this subroutine when first trying to run the motor and the
	motor is previously stopped. RunMotor will charge bootstrap caps,
	will initialize application variables, and will enable all ISRs.
*********************************************************************/

void RunMotor(void)
{
	ChargeBootstraps();
	// init variables
	
	TMR1 = 0;			// Reset timer 1 for speed control
	TMR3 = 0;			// Reset timer 3 for speed measurement
	ActualCapture = MAXPERIOD; // Initialize captures for minimum speed (60 RPMs)
	PastCapture = 0;
	Period = MAXPERIOD;
	while (IC7CONbits.ICBNE)
		IC7BUF;

	// Initialize direction with required direction
	// Remember that ADC is not stopped.
	HallValue = (unsigned int)((PORTB >> 3) & 0x0007);	// Read halls
	LastSector = Sector = SectorTable[HallValue];	// Initialize Sector variable

	// RefSpeed's sign will determine if the motor should be run at CW (+RefSpeed) or CCW (-RefSpeed) ONLY
	// at start up, since when the motor has started, the required direction will be set by the
	// control output variable to be able to operate in the four quadrants
	if (RefSpeed > 0)
	{
		ControlOutput = 0;
		Current_Direction = Required_Direction = CW;
		Speed = MINABSSPEED;
	}
	else
	{
		ControlOutput = 0;
		Current_Direction = Required_Direction = CCW;
		Speed = -MINABSSPEED;
	}
	SpeedAbsValue = MINABSSPEED;

	MotorStalledCounter = 0;	// Reset motor stalled protection counter

	// Clear all interrupts flags
	IFS0bits.T1IF = 0;	// Clear timer 1 flag
	IFS0bits.T2IF = 0;	// Clear timer 2 flag
	IFS0bits.CNIF = 0;	// Clear interrupt flag
	IFS1bits.IC7IF = 0;	// Clear interrupt flag
	IFS1bits.IC8IF = 0;	// Clear interrupt flag

	// enable all interrupts
	__asm__ volatile("DISI #5");
	IEC0bits.T1IE = 1;	// Enable interrupts for timer 1
	IEC0bits.CNIE = 1;	// Enable interrupts on CN5
	IEC1bits.IC7IE = 1;	// Enable interrupts on IC7
	IEC1bits.IC8IE = 1;	// Enable interrupts on IC8

	Flags.MotorRunning = 1;	// Indicate that the motor is running
	Flags.MotorCommInAdv = 0;
	Flags.MotorAdvEnabled = 0;
	return;
}

/*********************************************************************
	Call this subroutine whenever the user want to stop the motor.
	This will clear interrupts properly, and will also turn OFF all
	PWM channels.
*********************************************************************/

void StopMotor(void)
{
	OVDCON = 0x0000;	// turn OFF every transistor
	
	// disable all interrupts
	__asm__ volatile("DISI #5");
	IEC0bits.T1IE = 0;	// Disable interrupts for timer 1
	IEC0bits.T2IE = 0;	// Disable interrupts for timer 2
	IEC0bits.CNIE = 0;	// Disable interrupts on CN5
	IEC1bits.IC7IE = 0;	// Disable interrupts on IC7
	IEC1bits.IC8IE = 0;	// Disable interrupts on IC8

	Flags.MotorRunning = 0;	// Indicate that the motor has been stopped
	Flags.MotorCommInAdv = 0;
	return;
}

/********************************************************************
   This subroutine will calculate a timer value to commutate in advance.
   The value calculated will depend on the motor actual speed, so that the more
   Speed we have, the more phase advance we generate, up to 60 Degrees of Advance

                   |Actual Speed| - Minimum_Speed_Ph_Adv
   Timer Value = ----------------------------------------- x (60 Deg Time)
                            |Actual Speed|

*********************************************************************/

void PhaseAdvance(void)
{

	Flags.MotorCommInAdv = 0; // Indicate that a commutation in advanced has not occured
	T2CONbits.TON = 0;
	IFS0bits.T2IF = 0;
	IEC0bits.T2IE = 1;
	TMR2 = 0;
	Phase_Advance_Value = Period / 3;	// Take 60 Deg time information
	if (SpeedAbsValue > MIN_SPD_PH_ADV)	// If we are above the minimum ph adv speed
	{
		Phase_Advance_Perc_Dividend = SpeedAbsValue - (MIN_SPD_PH_ADV + 1);
		Phase_Advance_Perc_Divisor = SpeedAbsValue;
		Phase_Adv_Profile();
	}
	else
		PR2 = Phase_Advance_Value; // Load calculated value into timer register

	T2CONbits.TON = 1;

	return;	
}

/********************************************************************
	This subroutine implements the six-step commutation of a BLDC motor.
	Depending on the Sector the motor is in, the corresponding pair of
	transistors is energized based on a look up table. The duty cycle
	of the transistor with PWM is taken from the second parameter.
*********************************************************************/

void SixStepComm (int _Sector, int _Voltage)
{
	IEC0bits.T2IE = 0;
	IFS0bits.T2IF = 0;
	
	if (_Voltage >= 0)
	{
		// Load the same duty cycle to all the PDC registers.
		// The appropriate pair of transistors will be selected using
		// the overide control logic of the MCPWM module (OVDCON)
		PDC1 = PDC2 = PDC3 = (unsigned int)_Voltage / 16;
		if (_Sector == -1)
			OVDCON = 0x0000; // Invalid sector will shut down all transistors
		else
			OVDCON = StateLoTable[_Sector];
	}
	else
	{
		// Load the same duty cycle to all the PDC registers.
		// The appropriate pair of transistors will be selected using
		// the overide control logic of the MCPWM module (OVDCON)
		PDC1 = PDC2 = PDC3 = (unsigned int)(-(_Voltage+1)) / 16;
		if (_Sector == -1)
			OVDCON = 0x0000; // Invalid sector will shut down all transistors
		else
			OVDCON = StateLoTable[(_Sector + 3) % 6];
	}
	return;
}

/*********************************************************************
	This function is called each time the motor doesn't generate
	hall change interrupt, which means that the motor running too slow
	or is stalled. If it is stalled, the motor is stopped, but if it
	is only slow, this function is called and forces a commutation
	based on the actual hall sensor position and the required
	direction of rotation.
*********************************************************************/

void ForceCommutation(void)
{
	HallValue = (unsigned int)((PORTB >> 3) & 0x0007);	// Read halls
	Sector = SectorTable[HallValue];	// Read sector based on halls
	if (Sector != -1)	// If the sector is invalid don't do anything
	{
		// Depending on the required direction, a new phase is fetched
		if (Required_Direction == CW)
		{
			SixStepComm(Sector, ControlOutput);
		}
		else
		{
			SixStepComm(Sector, -(ControlOutput + 1));
		}
	}
	return;
}

/*******************************************************************
	Below is the code required to setup the ADC registers for :
	1. 1 channel conversion (in this case RB2/AN2)
	2. PWM trigger starts conversion
	3. Pot is connected to CH0 and RB2
	4. Manual Stop Sampling and start converting
	5. Manual check of Conversion complete 
	6. The data format will be signed fractional			
*********************************************************************/

void InitADC10(void)
{
	ADPCFG = 0xFFF8;				// RB3, RB4, and RB5 are digital
	ADCON1 = 0x026E;				// PWM starts conversion
									// Unsigned fractional conversions
	ADCON2 = 0x0000;
	ADCHS = 0x0002;					// Pot is connected to AN2

	ADCON3 = 0x0003;
	IFS0bits.ADIF = 0;				// Clear ISR flag
	IEC0bits.ADIE = 1;				// Enable interrupts
	
	ADCON1bits.ADON = 1;			// turn ADC ON
	return;
}

/********************************************************************
	InitMCPWM, intializes the PWM as follows:
	1. FPWM = 20000 hz
	2. Independent PWMs with Edge aligned
	3. Set Duty Cycle to 0 for Independent
	4. Set ADC to be triggered by PWM special trigger
*********************************************************************/

void InitMCPWM(void)
{
	TRISE = 0x0100;		// PWM pins as outputs, and FLTA as input
	PTPER = FCY/FPWM - 1;	// Compute Period based on CPU speed and required PWM frequency (see defines)
	OVDCON = 0x0000;	// Disable all PWM outputs.
	PWMCON1 = 0x0777;	// Enable PWM output pins and configure them as complementary mode		 
	PDC1 = 0;			// Initialize as 0 voltage
	PDC2 = 0;			// Initialize as 0 voltage
	PDC3 = 0;			// Initialize as 0 voltage
	SEVTCMP = PTPER;	// Enable triggering for ADC
	PWMCON2 = 0x0F00;	// 16 postscale values, for achieving 20 kHz
	PTCON = 0x8000;		// start PWM as edge aligned mode
	return;				 
}

/********************************************************************
	Configure Hall sensor inputs, one change notification and two
	input captures. on IC7 the actual capture value is used for
	further period calculation
*********************************************************************/

void Init_IC_and_CN(void)
{
	//Hall A -> CN5. Hall A is only used for commutation.
	//Hall B -> IC7. Hall B is used for Speed measurement and commutation.
	//Hall C -> IC8. Hall C is only used for commutation.
	
	// Init Input change notification 5
	TRISB |= 0x38;	// Ensure that hall connections are inputs 
	CNPU1 = 0;	// Disable all CN pull ups
	CNEN1 = 0x20;	// Enable CN5
	IFS0bits.CNIF = 0;	// Clear interrupt flag

	// Init Input Capture 7
	IC7CON = 0x0001;	// Input capture every edge with interrupts and TMR3
	IFS1bits.IC7IF = 0;	// Clear interrupt flag

	// Init Input Capture 8
	IC8CON = 0x0001;	// Input capture every edge with interrupts and TMR3
	IFS1bits.IC8IF = 0;	// Clear interrupt flag

	return;
}

/********************************************************************
	Initialization of timer 1 as a periodic interrupt each 1 ms for
	speed control, motor stalled protection which includes: forced
	commutation if the motor is too slow, or motor stopped if the
	motor is stalled.
*********************************************************************/

void InitTMR1(void)
{
	T1CON = 0x0020;			// internal Tcy/64 clock
	TMR1 = 0;
	PR1 = 313;				// 1 ms interrupts for 20 MIPS
	T1CONbits.TON = 1;		// turn on timer 1 
	return;
}

/********************************************************************
	Timer 2 will be used to schedule commutations when Phase Advance
	is enabled
*********************************************************************/

void InitTMR2(void)
{
	T2CON = 0x0020;
	TMR2 = 0;
	PR2 = 0;
}

/************************************************************************
	Initialization of timer 3 as the timebase for the capture channels
	for calculating the period of the halls.
*************************************************************************/

void InitTMR3(void)
{
	T3CON = 0x0020;			// internal Tcy/64 clock
	TMR3 = 0;
	PR3 = 0xFFFF;
	T3CONbits.TON = 1;		// turn on timer 3 
	return;
}

/************************************************************************
	Initialization of the IOs used by the application. The IOs for the
	PWM and for the ADC are initialized in their respective peripheral
	initialization subroutine.
*************************************************************************/

void InitUserInt(void)
{
	TRISC |= 0x4000;	// S2/RC14 as input
	// Analog pin for POT already initialized in ADC init subroutine
	PORTF = 0x0008;		// RS232 Initial values
	TRISF = 0xFFF7;		// TX as output
	return;
}
